@@ -373,6 +373,7 @@ class NoiseModelLayer(Layer):
                  info_for_layers_dict,
                  polynomial_order=2,
                  eta_regularization=0.01,
+                 is_noise_model_empirical=False,
                  **kwargs):
         """Construct class instance."""
         # order of polynomial which defines log_sigma's dependence on y_hat
@@ -380,6 +381,7 @@ class NoiseModelLayer(Layer):
         self.eta = eta_regularization
         self.info_for_layers_dict = info_for_layers_dict
         self.regularizer = tf.keras.regularizers.L2(self.eta)
+        self.is_noise_model_empirical = is_noise_model_empirical
         super().__init__(**kwargs)
 
     def get_config(self):
@@ -408,15 +410,40 @@ class NoiseModelLayer(Layer):
         -------
             A (B,) tensor containing negative log likelihood contributions
         """
-        # this is yhat
-        yhat = inputs[:, 0:1]
 
-        # these are the labels
-        ytrue = inputs[:, 1:]
+        # variable represent user supplied error bars, will get
+        # instantiated if noise model is empirical
+        self.dy = None
 
-        # Compute negative log likelihood
-        nlls = self.compute_nlls(yhat=yhat,
-                                 ytrue=ytrue)
+        # if noise model is not empirical, do everything as before
+        # i.e., do not supply the additional argument of dy to compute nll
+        if self.is_noise_model_empirical == False:
+            # this is yhat
+            yhat = inputs[:, 0:1]
+
+            # these are the labels
+            ytrue = inputs[:, 1:]
+
+            # Compute negative log likelihood
+            nlls = self.compute_nlls(yhat=yhat,
+                                     ytrue=ytrue)
+
+        # if noise model is empirical, supply argument dy to compute nll
+        # method in the Gaussian noise model derived class.
+        else:
+            # this is yhat
+            yhat = inputs[:, 0:1]
+
+            # these are the labels
+            ytrue = inputs[:, 1:2]
+
+            # these are user supplied error bars
+            self.dy = inputs[:,2:3]
+
+            # Compute negative log likelihood
+            nlls = self.compute_nlls(yhat=yhat,
+                                     ytrue=ytrue,
+                                     dy=self.dy)
 
         # Compute I_var metric from nlls
         H_y = self.info_for_layers_dict.get('H_y_norm', np.nan)
@@ -539,6 +566,83 @@ class GaussianNoiseModelLayer(NoiseModelLayer):
         return sigma
 
 
+class EmpiricalGaussianNoiseModelLayer(NoiseModelLayer):
+    """Represents an Empirical Gaussian noise model for GE regression."""
+
+    def __init__(self, *args, **kwargs):
+        """Construct layer instance."""
+        super().__init__(*args, **kwargs)
+
+    def build(self, input_shape):
+        """Build layer."""
+        self.a = self.add_weight(name='a',
+                                 shape=(self.K + 1, 1),
+                                 initializer="TruncatedNormal",
+                                 trainable=True,
+                                 regularizer=self.regularizer)
+        super().build(input_shape)
+
+    def get_config(self):
+        """Get configuration dictionary."""
+        base_config = super().get_config()
+        return {**base_config, "a": self.a}
+
+    @handle_arrays
+    def compute_params(self, yhat):
+        """Compute layer parameters governing p(y|yhat)."""
+
+        # Have to treat 0'th order term separately because of NaN bug.
+        logsigma = self.a[0]
+
+        # Add higher order terms and return
+        for k in range(1, self.K + 1):
+            logsigma += self.a[k] * K.pow(yhat, k)
+
+        return logsigma
+
+    @handle_arrays
+    def compute_nlls(self, yhat, ytrue, dy, use_arrays=False):
+        """
+        Compute negative log likelihood contributions for each datum.
+        Note: User must pass dy, since noise model is 'Empirical'
+        """
+        # Compute logsigma and sigma
+
+        # Set sigma equal to dy (user supplied error bars. )
+        logsigma = Log(dy)
+
+        # Compute nlls
+        nlls = \
+            0.5 * K.square((ytrue - yhat) / dy) \
+            + logsigma \
+            + 0.5*np.log(2*pi)
+
+        return nlls
+
+    # @handle_arrays
+    # def yhat_to_yq(self, yhat, q, dy):
+    #     """
+    #     Compute quantiles for p(y|yhat).
+    #     Note: User must pass dy, since noise model is 'Empirical'
+    #     """
+    #
+    #     # Set sigma equal to dy (user supplied error bars. )
+    #     sigma = dy.numpy().reshape(-1, 1)
+    #
+    #     yq = yhat + sigma * np.sqrt(2) * erfinv(2 * q.numpy() - 1)
+    #     return yq
+
+    @handle_arrays
+    def yhat_to_ymean(self, yhat):
+        """Compute mean of p(y|yhat)."""
+        return yhat
+
+    # @handle_arrays
+    # def yhat_to_ystd(self, yhat, dy):
+    #     """This method is dumb because the user supplies dy, which is ystd."""
+    #     return dy
+
+
 class CauchyNoiseModelLayer(NoiseModelLayer):
     """Represents a Cauchy noise model for GE regression."""
 
