@@ -17,6 +17,7 @@ from mavenn.src.layers.measurement_process_layers \
     import GlobalEpistasisLayer, \
         AffineLayer, \
         GaussianNoiseModelLayer, \
+        EmpiricalGaussianNoiseModelLayer, \
         CauchyNoiseModelLayer, \
         SkewedTNoiseModelLayer, \
         MPAMeasurementProcessLayer
@@ -66,6 +67,10 @@ class GlobalEpistasisModel:
 
     eta_regularization: (float >= 0)
         Regularization strength for measurement process parameters eta.
+
+    initial_weights: (np.array)
+        Numpy array of weights that gets set as initial weights of a model
+        if not set to None.
     """
 
     @handle_errors
@@ -81,7 +86,8 @@ class GlobalEpistasisModel:
                  ge_heteroskedasticity_order,
                  theta_regularization,
                  eta_regularization,
-                 custom_gpmap):
+                 custom_gpmap,
+                 initial_weights):
         """Construct class instance."""
         # set class attributes
         self.info_for_layers_dict = info_for_layers_dict
@@ -96,6 +102,7 @@ class GlobalEpistasisModel:
         self.eta_regularization = eta_regularization
         self.ge_nonlinearity_type = ge_nonlinearity_type
         self.custom_gpmap = custom_gpmap
+        self.initial_weights = initial_weights
 
         # class attributes that are not parameters
         # but are useful for using trained models
@@ -177,7 +184,7 @@ class GlobalEpistasisModel:
 
         ge_noise_model_type: (str)
             Specifies the type of noise model the user wants to infer.
-            The possible choices allowed: ['Gaussian','Cauchy','SkewedT']
+            The possible choices allowed: ['Gaussian','Cauchy','SkewedT', 'Empirical']
 
         Returns
         -------
@@ -185,7 +192,7 @@ class GlobalEpistasisModel:
             TensorFlow model that can be compiled and subsequently fit to data.
         """
         # check that p_of_all_y_given_phi valid
-        check(ge_noise_model_type in {'Gaussian', 'Cauchy', 'SkewedT'},
+        check(ge_noise_model_type in {'Gaussian', 'Cauchy', 'SkewedT', 'Empirical'},
               f'p_of_all_y_given_phi = {ge_noise_model_type};' 
               f'must be "Gaussian", "Cauchy", or "SkewedT"')
 
@@ -198,7 +205,13 @@ class GlobalEpistasisModel:
         # Compute number of sequence nodes. Useful for model construction below.
         number_x_nodes = int(self.L*self.C)
 
-        number_input_layer_nodes = number_x_nodes + 1
+        # need to add one additional node for targets
+        # but 2 additional nodes if noise model is empirical; 1 for y
+        # and another for dy
+        if ge_noise_model_type != 'Empirical':
+            number_input_layer_nodes = number_x_nodes + 1
+        else:
+            number_input_layer_nodes = number_x_nodes + 2
 
         inputTensor = Input((number_input_layer_nodes,),
                             name='Sequence_labels_input')
@@ -211,6 +224,14 @@ class GlobalEpistasisModel:
             output_shape=((1,)),
             trainable=False, name='Labels_input')(inputTensor)
 
+        # User supplied error bars
+        if ge_noise_model_type == 'Empirical':
+
+            dlabels_input = Lambda(
+                lambda x: x[:, number_x_nodes+1:number_x_nodes + 2],
+                output_shape=((1,)),
+                trainable=False, name='dLabels_input')(inputTensor)
+
 
         # Create G-P map layer
         if self.gpmap_type == 'additive':
@@ -251,9 +272,16 @@ class GlobalEpistasisModel:
 
         yhat = self.phi_to_yhat_layer(phi)
 
-        # Concatenate yhat and training labels
-        yhat_y_concat = Concatenate(name='yhat_and_y_to_ll')(
-            [yhat, labels_input])
+        # Concatenate yhat and training labels only if
+        # noise model is not empirical, other concatenate
+        # user error bars also.
+        if ge_noise_model_type != 'Empirical':
+
+            yhat_y_concat = Concatenate(name='yhat_and_y_to_ll')(
+                [yhat, labels_input])
+        else:
+            yhat_y_concat = Concatenate(name='yhat_and_y_to_ll')(
+                [yhat, labels_input, dlabels_input])
 
         # Create noise model layer
         if ge_noise_model_type == 'Gaussian':
@@ -273,6 +301,13 @@ class GlobalEpistasisModel:
                 info_for_layers_dict=self.info_for_layers_dict,
                 polynomial_order=self.ge_heteroskedasticity_order,
                 eta_regularization=self.eta_regularization)
+
+        elif ge_noise_model_type == 'Empirical':
+            self.noise_model_layer = EmpiricalGaussianNoiseModelLayer(
+                info_for_layers_dict=self.info_for_layers_dict,
+                polynomial_order=self.ge_heteroskedasticity_order,
+                eta_regularization=self.eta_regularization,
+                is_noise_model_empirical=True)
         else:
             assert False, 'This should not happen.'
 
@@ -283,6 +318,14 @@ class GlobalEpistasisModel:
         self.model = model
         self.ge_nonlinearity_hidden_nodes = ge_nonlinearity_hidden_nodes
 
+        # set initial weights if provided.
+        if self.initial_weights is not None:
+
+            # check(isinstance(self.initial_weights, np.ndarray),
+            #       f'type(initial_weights)={type(self.initial_weights)}; must be a numpy array.')
+
+            model.set_weights(self.initial_weights)
+
         return model
 
 
@@ -322,6 +365,10 @@ class MeasurementProcessAgnosticModel:
     custom_gpmap: (GPMapLayer sub-class)
         Defines custom gpmap, provided by user. Inherited class of GP-MAP layer,
         which defines the functionality for x_to_phi_layer.
+
+    initial_weights: (np.array)
+        Numpy array of weights that gets set as initial weights of a model
+        if not set to None.
     """
 
     @handle_errors
@@ -335,7 +382,8 @@ class MeasurementProcessAgnosticModel:
                  theta_regularization,
                  eta_regularization,
                  ohe_batch_size,
-                 custom_gpmap):
+                 custom_gpmap,
+                 initial_weights):
         """Construct class instance."""
         # set class attributes
         self.info_for_layers_dict = info_for_layers_dict
@@ -348,6 +396,7 @@ class MeasurementProcessAgnosticModel:
         self.ohe_batch_size = ohe_batch_size
         self.number_of_bins = number_of_bins
         self.custom_gpmap = custom_gpmap
+        self.initial_weights = initial_weights
 
         # class attributes that are not parameters
         # but are useful for using trained models
@@ -481,4 +530,13 @@ class MeasurementProcessAgnosticModel:
         model = Model(inputTensor, outputTensor)
         self.model = model
         self.mpa_hidden_nodes = mpa_hidden_nodes
+
+        # set initial weights if provided.
+        if self.initial_weights is not None:
+
+            # check(isinstance(self.initial_weights, np.ndarray),
+            #      f'type(initial_weights)={type(self.initial_weights)}; must be a numpy array.')
+
+            model.set_weights(self.initial_weights)
+
         return model
