@@ -60,6 +60,9 @@ def load(filename, verbose=True):
         loaded_model.info_for_layers_dict = \
             config_dict.get('info_for_layers_dict', {})
 
+        #loaded_model.set_data_args = config_dict.get('set_data_args')
+        loaded_model.fit_args = config_dict.get('fit_args')
+
         # Load and set weights
         filename_h5 = filename + '.h5'
         loaded_model.get_nn().load_weights(filename_h5)
@@ -436,15 +439,16 @@ def x_to_stats(x, alphabet, weights=None, verbose=False):
 
 
 @handle_errors
-def set_seed(seed):
+def set_seed(seed=1234):
     """
     Set random number generator seed; use to make training reproducible.
 
     Parameters
     ----------
     seed: (int)
-        The value provided is used in both np.random.seed()
-        and tf.random.set_seed().
+        The value provided is used in np.random.seed(),
+        tf.random.set_seed(), random.seed() and
+        environ['PYTHONHASHSEED']. Default is seed=1234.
 
     Returns
     -------
@@ -455,10 +459,9 @@ def set_seed(seed):
           f'type(seed)={type(seed)}; must be int')
 
     # Set seed
-    os.environ['PYTHONHASHSEED'] = '0'
-    tf.random.set_seed(seed)
-    np.random.seed(seed)
     random.seed(seed)
+    np.random.seed(seed)
+    tf.random.set_seed(seed)
 
 
 @handle_errors
@@ -525,7 +528,127 @@ def x_to_ohe(x,
 
     return x_ohe
 
+
+def only_single_mutants(training_sequences, consensus_sequence, alphabet):
+    """
+    Method that checks if training data contains only single mutants or not.
+
+    training_sequences: (array)
+        Array of sequences used for training models.
+
+    consensus_sequence:
+        Consensus sequence, evaluated using training data.
+        Available as model.x_to_stats['consensus_seq'].
+
+    alphabet: (np.array)
+        The alphabet, length C, from which sequences will be generated.
+
+    returns
+    -------
+    only_single_mutants_in_training_data: (bool)
+        True if only single mutants founds compared to consensus
+        sequence, false otherwise.
+    """
+
+    # Convert the consensus and training sequences to one-hot encoded.
+    consensus_seq_ohe = x_to_ohe(consensus_sequence, alphabet)
+    training_seqs_ohe = x_to_ohe(training_sequences, alphabet)
+
+    # Calculate the hamming distance between all the training
+    # sequences and the consensus sequence and store it in
+    # hamming_dist array.
+    hamming_dist = np.sum(np.bitwise_xor(
+        training_seqs_ohe, consensus_seq_ohe), axis=1)/2.0
+
+    # If all the elements of hamming_dist array are less than
+    # and equal to one, then training set consists of
+    # only single mutants.
+    only_single_mutants_in_training_data = np.all(hamming_dist <= 1)
+
+    return only_single_mutants_in_training_data
+
+
 # Converts sequences to matrices
 def _x_to_mat(x, alphabet):
     return (np.array(list(x))[:, np.newaxis] ==
             alphabet[np.newaxis, :]).astype(float)
+
+@handle_errors
+def split_dataset(data_df,
+                  set_col='set',
+                  train_set_name='training',
+                  val_set_name='validation',
+                  test_set_name='test',
+                  verbose=True):
+    """
+    Splits dataset into
+        (1) `trainval_df`: training + validation set
+        (2) `train_df`: test set
+    based on the value of the column `set_col`, which is then dropped. Also
+    adds a `val_set_name` column to trainval_df indicating which data is to be
+    reserved for validation (as opposed to gradient descent).
+
+    Parameters
+    ----------
+    data_df: (pd.DataFrame)
+        Dataset to split
+
+    set_col: (str)
+        Column of data_df indicating training, validation, or test set
+
+    train_set_name: (str)
+        Value in data_df[set_col] indicating allocation to training set
+
+    val_set_name: (str)
+        Value in data_df[set_col] indicating allocation to validation set
+
+    test_set_name: (str)
+        Value in data_df[set_col] indicating allocation to test set
+
+    verbose: (bool)
+        Whether to print out the size of the resulting datasets
+
+    Returns
+    -------
+    trainval_df: (pd.DataFrame)
+        Training + validation dataset. Contains a column named `val_set_name`
+        indicating whether a row is allocated to the training or validation
+        set.
+
+    test_df: (pd.DataFrame)
+        Test dataset.
+    """
+
+    # Specify training + validation sets
+    trainval_ix = data_df[set_col].isin([train_set_name, val_set_name])
+    trainval_df = data_df[trainval_ix].copy().reset_index(drop=True)
+    trainval_df.insert(loc=0,
+                       column=val_set_name,
+                       value=trainval_df[set_col].eq(val_set_name))
+    trainval_df.drop(columns=set_col, inplace=True)
+
+    # Specify test set
+    test_ix = data_df[set_col].eq(test_set_name)
+    test_df = data_df[test_ix].copy().reset_index(drop=True)
+    test_df.drop(columns=set_col, inplace=True)
+
+    # Show dataset sizes
+    if verbose:
+        # Print number of data points
+        num_data = len(data_df)
+        num_train = sum(~trainval_df["validation"])
+        pct_train = 100 * num_train / num_data
+        num_val = sum(trainval_df["validation"])
+        pct_val = 100 * num_val / num_data
+        num_test = len(test_df)
+        pct_test = 100 * num_test / num_data
+
+        print(f'Training set   : {num_train:8,d} observations ({pct_train:7.2f}%)')
+        print(f'Validation set : {num_val:8,d} observations ({pct_val:7.2f}%)')
+        print(f'Test set       : {num_test:8,d} observations ({pct_test:7.2f}%)')
+        print('-------------------------------------------------')
+        print(f'Total dataset  : {num_data:8,d} observations ({100:7.2f}%)')
+        print('')
+
+    # return
+    return trainval_df, test_df
